package at.xaxa.ledger.ui.category.add

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.filled.Info
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.data.db.Category.CategoryEntity
import at.xaxa.ledger.ui.AppViewModelProvider
import at.xaxa.ledger.ui.ButtonSuccess
import at.xaxa.ledger.ui.category.iconNames
import at.xaxa.ledger.ui.category.icons


@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun AddCategory(
    onButtonClick: () -> Unit,
    modifier: Modifier = Modifier,
    categoryViewModel: AddCategoryViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {

    var name by remember { mutableStateOf("") }
    var selectedIconIndex by remember { mutableStateOf(-1) } // Store index of selected icon
    var expanded by remember { mutableStateOf(false) } // Controls dropdown visibility
    var parsingError by remember { mutableStateOf(false) }

    if (parsingError) {
        AlertDialog(
            icon = {
                Icon(Icons.Default.Info, contentDescription = "Info Icon")
            },
            title = {
                Text(text = "Parsing error")
            },
            text = {
                Text(text = "One or more fields contain invalid input. Please review your entries and try again.")
            },
            onDismissRequest = {
                parsingError = false
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        parsingError = false
                    }
                ) {
                    Text("Okay")
                }
            }
        )
    }

    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp, 0.dp),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Column(
            Modifier.weight(1f)
        ) {
            OutlinedTextField(
                value = name,
                onValueChange = { name = it },
                label = { Text("Category Name") },
                modifier = Modifier.fillMaxWidth().verticalScroll(rememberScrollState()),
                maxLines = 3,
            )

            ExposedDropdownMenuBox(
                expanded = expanded,
                onExpandedChange = { expanded = it }
            ) {
                OutlinedTextField(
                    value =
                        if (selectedIconIndex == -1){
                            ""
                        }else{
                            iconNames[selectedIconIndex]
                        }
                    ,
                    onValueChange = {},
                    label = { Text("Icon") },
                    readOnly = true,
                    leadingIcon = {
                        if (selectedIconIndex != -1){
                            Icon(
                                imageVector = icons[selectedIconIndex], // Replace with your desired icon
                                contentDescription = "Leading Icon"
                            )
                        }
                    },
                    trailingIcon = {
                        ExposedDropdownMenuDefaults.TrailingIcon(expanded = expanded)
                    },
                    modifier = Modifier
                        .menuAnchor()
                        .fillMaxWidth()
                )

                ExposedDropdownMenu(
                    expanded = expanded,
                    onDismissRequest = { expanded = false }
                ) {
                    icons.forEachIndexed { index, icon ->
                        DropdownMenuItem(
                            text = { Text(text = iconNames[index]) }, // Use name from iconNames
                            onClick = {
                                selectedIconIndex = index // Update selected index
                                expanded = false
                            },
                            leadingIcon = {
                                androidx.compose.material3.Icon(
                                    imageVector = icon,
                                    contentDescription = iconNames[index]
                                )
                            }
                        )
                    }
                }
            }
        }
        Box(
            modifier = Modifier
                .fillMaxWidth(),
            contentAlignment = Alignment.Center
        ) {
            ButtonSuccess(
                modifier = Modifier,
                "Add Category",
                onClick = {
                    if (name.isNotBlank() && selectedIconIndex != -1) {
                        val newCategory = CategoryEntity(
                            _id = 0,
                            categoryName = name,
                            icon = selectedIconIndex // Save selected index as the icon ID
                        )

                        categoryViewModel.addCategory(newCategory)
                        onButtonClick()
                    }else{
                        parsingError = true
                    }
                }
            )
        }
    }
}
